#!/usr/local/bin/perl -w

=head1 backupdb.pl
    
    History:
        BWK 23 May 00 -- Written; Bill Kelly <billk@cts.com>

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    Copyright (C) 2000 Bill Kelly / Full Body Groove.  All Rights Reserved.
    This software is released in the hope it will be useful, but please
    understand you use it at your own risk.
    THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
    WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
    OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

=cut

BEGIN { umask(000); open (STDERR, ">>qwiki-log"); }
print STDERR (scalar(localtime)." $0 for ".($ENV{'HTTP_CLIENT_ADDR'} || $ENV{'REMOTE_ADDR'})."\n");

use strict;
require FSDBM_File;
require FulltextIndexer_Adaptor;
require porter;  # word stemmer
require qwcfg;  # QwikWiki config

$| = 1;  # enable autoflush
print "Content-type:text/html\n\n<pre>\n";

# stupidity to quell damnable 'only used once' warning :-((
$dev::null = $qwcfg::pagesDbPath . $qwcfg::revsDbPath
           . $qwcfg::searchDbPath . $qwcfg::stopwordsFile;
undef $dev::null;
# end stupidity

%qwdbm::pages = ();
%qwdbm::revs = ();
%qwdbm::searchDb = ();
%qwdbm::search = ();
tie(%qwdbm::pages, 'FSDBM_File', $qwcfg::pagesDbPath);
tie(%qwdbm::revs, 'FSDBM_File', $qwcfg::revsDbPath);
tie(%qwdbm::searchDb, 'FSDBM_File', $qwcfg::searchDbPath);
tie(%qwdbm::search, 'FulltextIndexer_Adaptor',
    \%qwdbm::searchDb, 1, $qwcfg::stopwordsFile, \&porter::porter);

my $bak     = "db.bak.tar";
my $bak2    = "db.bak2.tar";
my $bak_gz  = "$bak.gz";
my $bak2_gz = "$bak2.gz";

chdir $qwcfg::qwikiProgRoot or die "can't chdir to $qwcfg::qwikiProgRoot! Aborting. ($!)";

if (-f $bak_gz) {
    print "Preserving previous backup as $bak2_gz...\n";
    unlink $bak2_gz if (-f $bak2_gz);
    rename ($bak_gz, $bak2_gz) or die "rename ($bak_gz, $bak2_gz) failed! Aborting. ($!)";
}

print "Locking all databases...\n";
tied(%qwdbm::pages)->LockDb();
tied(%qwdbm::revs)->LockDb();
tied(%qwdbm::search)->LockDb();  # locking search handles locking its searchDb

print "Tarring to $bak...\n";
`tar cf $bak db`;

print "Unlocking all databases...\n";
tied(%qwdbm::search)->UnlockDb();
tied(%qwdbm::revs)->UnlockDb();
tied(%qwdbm::pages)->UnlockDb();

print "Zipping to $bak_gz...\n";
`gzip $bak`;

print "Finished.\n";

untie(%qwdbm::search);
untie(%qwdbm::searchDb);
untie(%qwdbm::revs);
untie(%qwdbm::pages);


# eof

