
=head1 QwikWiki diff

    History:
        BWK 14 May 00 -- Written; Bill Kelly <billk@cts.com>

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    Copyright (C) 2000 Bill Kelly / Full Body Groove.  All Rights Reserved.
    This software is released in the hope it will be useful, but please
    understand you use it at your own risk.
    THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
    WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
    OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

=cut

package qwdiff;
use strict;
use IO::Handle;
use Carp;
require qwcfg;  # QwikWiki config

BEGIN {
    use Exporter   ();
    use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

    $VERSION     = 1.00;

    @ISA         = qw(Exporter);
    @EXPORT      = qw(&hilightPageDiffs);
    %EXPORT_TAGS = ( );
    @EXPORT_OK   = qw(%hilightOn %hilightOff);
}

use vars qw(%hilightOn %hilightOff);

%hilightOn = (
    NODIFF     => '',
    ADD        => '<!--qwdiff--><font style="color:black;background-color:#66ff66">',          # was: ffff66
    DELETE     => '<!--qwdiff--><font style="color:black;background-color:#ff99aa"><strike>',  # was: ff6666
    CHANGEFROM => '<!--qwdiff--><font style="color:black;background-color:#ff99aa"><strike>',  # was: ff9966
    CHANGETO   => '<!--qwdiff--><font style="color:black;background-color:#66ff66">'           # was: 66ff66
);

%hilightOff = (
    NODIFF     => '',
    ADD        => '<!--qwdiff_end--></font>',
    DELETE     => '<!--qwdiff_end--></strike></font>',
    CHANGEFROM => '<!--qwdiff_end--></strike></font>',
    CHANGETO   => '<!--qwdiff_end--></font>'
);


sub whowasi { (caller(1))[3] }

sub hilightPageDiffs {
    my ($oldText, $newText) = @_;

    my $splitOld = wordSplitDiffText($oldText);
    my $splitNew = wordSplitDiffText($newText);
    my $tmpFile1 = "$qwcfg::qwdiffTmpDir/qwdiff.$$.1";
    my $tmpFile2 = "$qwcfg::qwdiffTmpDir/qwdiff.$$.2";
    my $fh = IO::Handle->new();
    open($fh, ">$tmpFile1")
        or croak "Error: @{[&whowasi]} Couldn't open diff tmp file $tmpFile1 ($!)";
    print $fh ($splitOld);
    close($fh);
    open($fh, ">$tmpFile2")
        or croak "Error: @{[&whowasi]} Couldn't open diff tmp file $tmpFile2 ($!)";
    print $fh ($splitNew);
    close($fh);
    my $diffList = getDiffList($tmpFile1, $tmpFile2);
    unlink $tmpFile1, $tmpFile2;
    my $hiText = genHilightDiffHtml($splitOld, $splitNew, $diffList);
    return $hiText;
}

sub genHilightDiffHtml {
    my ($splitOld, $splitNew, $diffList) = @_;

    my @linesOld = split('\n', $splitOld);
    my @linesNew = split('\n', $splitNew);
    my $html = "";
    my $cur = 1;
    my $diff;
    foreach $diff (@$diffList) {
        yank(\$html, \@linesNew, $cur, $$diff{from2} - 1, 'NODIFF');
        $cur = $$diff{to2} + 1;
        if ($$diff{kind} eq 'c') {
            yank(\$html, \@linesOld, $$diff{from1}, $$diff{to1}, 'CHANGEFROM');
            yank(\$html, \@linesNew, $$diff{from2}, $$diff{to2}, 'CHANGETO');
        }
        elsif ($$diff{kind} eq 'a') {
            yank(\$html, \@linesNew, $$diff{from2}, $$diff{to2}, 'ADD');
        }
        elsif ($$diff{kind} eq 'd') {
            yank(\$html, \@linesNew, $$diff{from2}, $$diff{to2}, 'NODIFF');
            yank(\$html, \@linesOld, $$diff{from1}, $$diff{to1}, 'DELETE');
        }
    }
    yank(\$html, \@linesNew, $cur, scalar(@linesNew), 'NODIFF');
    return $html;
}

sub yank {
    my ($dest, $lines, $from, $to, $kind) = @_;
    
    $$dest .= $hilightOn{$kind};
    for (--$from, --$to ;  $from <= $to;  $from++) {
        my $line = $$lines[$from];
        # Form-feed was newline in original.  Hilighting is stopped and resumed
        # around newlines to simplify things in qwiki::_markupWikiPage,
        # which is line-based.
        $line =~ s/^[\014]/$hilightOff{$kind} . "\n" . $hilightOn{$kind}/e;
        $$dest .= $line;
    }
    $$dest .= $hilightOff{$kind};
}

sub wordSplitDiffText {
    my ($text) = @_;
    # Newlines in original are stashed as form-feeds.
    # New newlines are inserted at word boundaries for diffing.
    # The form-feeds may only appear as the first character these new "diff lines".
    # (This makes things easier for yank, and ultimately for qwiki::_markupWikiPage)
    $text =~ s/[\015][\012]/\014/gs;
    $text =~ s/[\015\012]/\014/gs;
    $text =~ s/( [\014]?[A-Za-z]+(?:[ ]+)? | [\014]?[^A-Za-z\014]+ | [\014](?:$|(?=[\014])) )/$1\n/gsx;
    return $text;
}

sub getDiffList {
    my ($file1, $file2) = @_;

    my $pipeh = IO::Handle->new();
    open($pipeh, "$qwcfg::diffProg $file1 $file2 |")
        or croak "Error: @{[&whowasi]} Failed to fork for pipe to $qwcfg::diffProg ($!)";
 # print "<hr><pre>\n";
    my $diffList = [];
    my $line;
    while (defined($line = <$pipeh>)) {
 # print $line;
        if ($line =~ m/^(\d+)(?:,(\d+))?([acd])(\d+)(?:,(\d+))?/) {
            my ($kind, $from1, $to1, $from2, $to2) = ($3, $1, $2, $4, $5);
 # local($^W) = undef;
 # print "got ($&) ($3) ($1) ($2) ($4) ($5)\n";
            $to1 = $from1 unless ($to1);
            $to2 = $from2 unless ($to2);
            push(@$diffList, { kind => $kind, from1 => $from1, to1 => $to1,
                                              from2 => $from2, to2 => $to2 });
        }
    
    }
    close($pipeh);
      # or carp "Error: @{[&whowasi]} Pipe close failed for $qwcfg::diffProg ($? $!)";
 # print "</pre>";
    return $diffList;
}




#   got (4c4) (c) (4) () (4) ()
#   got (145a146,197) (a) (145) () (146) (197)
#   got (403,454d454) (d) (403) (454) (454) ()
#   got (484,487c484,488) (c) (484) (487) (484) (488)


#   145a146,197
#   > This
#   ...
#   
#   
#   403,454d454
#   < This
#   ...
#   
#   
#   4c4
#   < recursive
#   ---
#   > recursive-matic
#   
#   
#   484,487c484,488
#   < Intransigent
#   < Extramarital
#   < Insurgencies
#   < Romantic
#   ---
#   > Spreading
#   > Activation
#   > Model
#   > of
#   > Semantic



#    diff s3y s3 | more
#   4c4
#   < recursive-matic
#   ---
#   > recursive
#   145a146,197
#   > This
#   > attenuation
#   > might
#   > be
#   > specific
#   > to
#   > the
#   > arc
#   > carrying
#   > the
#   > activation
#   > (
#   > e
#   > .
#   > g
#   > .,
#   > some
#   > arcs
#   > in
#   > the
#   > network
#   > might
#   > be
#   > more
#   > or
#   > less
#   > conductive
#   > than
#   > others
#   > ,
#   > reflecting
#   > their
#   > salience
#   > )
#   > or
#   > constant
#   > across
#   > the
#   > network
#   > (
#   > e
#   > .
#   > g
#   > .,
#   > traversing
#   > any
#   > arc
#   > causes
#   > 10
#   > %
#   > attentuation
#   > ).
#   403,454d454
#   < This
#   < attenuation
#   < might
#   < be
#   < specific
#   < to
#   < the
#   < arc
#   < carrying
#   < the
#   < activation
#   < (
#   < e
#   < .
#   < g
#   < .,
#   < some
#   < arcs
#   < in
#   < the
#   < network
#   < might
#   < be
#   < more
#   < or
#   < less
#   < conductive
#   < than
#   < others
#   < ,
#   < reflecting
#   < their
#   < salience
#   < )
#   < or
#   < constant
#   < across
#   < the
#   < network
#   < (
#   < e
#   < .
#   < g
#   < .,
#   < traversing
#   < any
#   < arc
#   < causes
#   < 10
#   < %
#   < attentuation
#   < ).
#   484,487c484,488
#   < Intransigent
#   < Extramarital
#   < Insurgencies
#   < Romantic
#   ---
#   > Spreading
#   > Activation
#   > Model
#   > of
#   > Semantic


# diff s3 s3y
#   4c4
#   < recursive
#   ---
#   > recursive-matic
#   146,197d145
#   < This
#   < attenuation
#   < might
#   < be
#   < specific
#   < to
#   < the
#   < arc
#   < carrying
#   < the
#   < activation
#   < (
#   < e
#   < .
#   < g
#   < .,
#   < some
#   < arcs
#   < in
#   < the
#   < network
#   < might
#   < be
#   < more
#   < or
#   < less
#   < conductive
#   < than
#   < others
#   < ,
#   < reflecting
#   < their
#   < salience
#   < )
#   < or
#   < constant
#   < across
#   < the
#   < network
#   < (
#   < e
#   < .
#   < g
#   < .,
#   < traversing
#   < any
#   < arc
#   < causes
#   < 10
#   < %
#   < attentuation
#   < ).
#   454a403,454
#   > This
#   > attenuation
#   > might
#   > be
#   > specific
#   > to
#   > the
#   > arc
#   > carrying
#   > the
#   > activation
#   > (
#   > e
#   > .
#   > g
#   > .,
#   > some
#   > arcs
#   > in
#   > the
#   > network
#   > might
#   > be
#   > more
#   > or
#   > less
#   > conductive
#   > than
#   > others
#   > ,
#   > reflecting
#   > their
#   > salience
#   > )
#   > or
#   > constant
#   > across
#   > the
#   > network
#   > (
#   > e
#   > .
#   > g
#   > .,
#   > traversing
#   > any
#   > arc
#   > causes
#   > 10
#   > %
#   > attentuation
#   > ).
#   484,488c484,487
#   < Spreading
#   < Activation
#   < Model
#   < of
#   < Semantic
#   ---
#   > Intransigent
#   > Extramarital
#   > Insurgencies
#   > Romantic



1;

# eof

