#!/usr/local/bin/perl -w

=head1 QwikWiki

    History:
        BWK  7 May 00 -- Begun; Bill Kelly <billk@cts.com>

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    Copyright (C) 2000 Bill Kelly / Full Body Groove.  All Rights Reserved.
    This software is released in the hope it will be useful, but please
    understand you use it at your own risk.
    THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
    WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
    OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

=cut


BEGIN { umask(000); open (STDERR, ">>qwiki-log"); }
print STDERR _getLogmsg("begin");
$SIG{'PIPE'} = 'IGNORE';
$| = 1;  # enable autoflush
print "Content-type:text/html\n\n";  # do this immediately - some http servers time out quickly?

use strict;
use CGI ':standard';
require FSDBM_File;
require FulltextIndexer_Adaptor;
require porter;  # word stemmer
require qwcfg;  # QwikWiki config
use qwiki;
print qq{<!--QwikWiki V$qwiki::VERSION-->\n};

$qwenv::goodExit = undef;  # END{} block prints err message if not set on exit
$qwenv::clientAddr = defined($ENV{'HTTP_CLIENT_ADDR'})?
                        $ENV{'HTTP_CLIENT_ADDR'} : $ENV{'REMOTE_ADDR'};
$qwenv::clientAddr = "unknown" unless($qwenv::clientAddr);
$qwenv::htmlOut = "";  # output appended here, and printed if script successful
$qwenv::referrerPage = _getReferrerPage();

logmsg("before tie");

%qwdbm::pages = ();
%qwdbm::revs = ();
%qwdbm::searchDb = ();
%qwdbm::search = ();
tie(%qwdbm::pages, 'FSDBM_File', $qwcfg::pagesDbPath);
tie(%qwdbm::revs, 'FSDBM_File', $qwcfg::revsDbPath);
tie(%qwdbm::searchDb, 'FSDBM_File', $qwcfg::searchDbPath);
tie(%qwdbm::search, 'FulltextIndexer_Adaptor',
    \%qwdbm::searchDb, 1, $qwcfg::stopwordsFile, \&porter::porter);

logmsg("before new CGI");

my $q = new CGI;
qwiki::qwikiInit($q);
# print $q->header;  (done at top of file before http server times out)

logmsg("before qwiki");

my $pageName = "";
if (defined($pageName = $q->param('FulltextSearch'))) {
    logmsg("do_search $pageName");
    fulltextSearch($pageName);
}
elsif (defined($pageName = $q->param('search')) ||
        (defined($q->param('keywords')) && ($q->param('keywords') eq 'search'))) {
    logmsg("search_form");
    searchForm($pageName);
}
elsif (defined($pageName = $q->param('edit'))) {
    logmsg("edit $pageName");
    editPage($pageName) unless detectNonWikiName($pageName);
}
elsif (defined($pageName = $q->param('copy'))) {
    logmsg("edit_copy $pageName");
    editCopy($pageName) unless detectNonWikiName($pageName);
}
elsif (defined($pageName = $q->param('diff'))) {
    logmsg("diff $pageName");
    diffPage($pageName) unless detectNonWikiName($pageName);
}
elsif (defined($pageName = $q->param('SavePage'))) {
    logmsg("save $pageName");
    savePage($pageName) unless detectNonWikiName($pageName);
}
else {
    $pageName = $q->param('keywords') || $qwcfg::FrontPage;
    logmsg("view $pageName");
    viewPage($pageName) unless detectNonWikiName($pageName);
}

logmsg("before untie");

untie(%qwdbm::search);
untie(%qwdbm::searchDb);
untie(%qwdbm::revs);
untie(%qwdbm::pages);

logmsg("before emit");

print $qwenv::htmlOut, $q->end_html;
$qwenv::goodExit = 1;


#
# Attempts to determine which Wiki page we came from, if any.  (Doesn't
# differentiate between wheter we were viewing, editing, diffing, etc.)
# Note: currently expects any leading query keywords (like search=, edit=, etc.)
# will begin with a lowercase a-z.
sub _getReferrerPage {
    my $url = $ENV{'HTTP_REFERER'};
    return "" unless ($url);
    if ($url =~ /$qwcfg::qwikiUrl(?:[?]?$|[?](?:[a-z]+\w*=)?((?:[A-Z][a-z]*){2,})\b)/so) {
        return ($1)? $1 : $qwcfg::FrontPage;
    }
    return "";
}

sub logmsg {
    my $msg = _getLogmsg(@_);
    print STDERR $msg;
}

sub _getLogmsg {
    my $remoteHost = defined($ENV{'HTTP_CLIENT_ADDR'})?
                        $ENV{'HTTP_CLIENT_ADDR'} : $ENV{'REMOTE_ADDR'};
    my $progName = $0;
    $progName =~ s{^.*/(.*)$}{$1};
    return "\[".(scalar localtime)." $progName:$$ for $remoteHost\] @_\n";
}


# Write script completion message to log file
END {
    if ($qwenv::goodExit) {
      # print qq{<p><font color="FFFFFF">QwikWiki V$qwiki::VERSION</font></p>\n};
      # print qq{<p><font color="FFFFFF">Please disregard the segfault, if any appears below.  This script was occasionally crashing Perl on exit somehow...<br>Exiting Perl now...\n};
    }
    else {
        print qq{<head><title>Wiki Server Error</title></head>\n};
        print qq{<h1><font color="FF0000">Wiki Server Error</font> <small>(QwikWiki V$qwiki::VERSION)</small></h1>\n};
        print qq{<h2>An Error Occurred While Processing Your Request</h2>\n};
        print qq{<p>This information has been logged.<br>\n};
        print qq{We are sorry for any inconvenience.</p>\n};
        print STDERR _getLogmsg("bad exit ($! $? $@)");
    }
    print STDERR _getLogmsg("end");
}


# show environment vars:
#   print br, hr, "<p><tt>";
#   my $key;
#   foreach $key (sort keys %ENV) {
#       my $evar = "$key = $ENV{$key}";
#       $evar =~ s/</&lt;/gs;
#       $evar =~ s/>/&gt;/gs;
#       print "$evar\n", br;
#   }
#   print "</tt></p>";

# show stdin:
#   my $l;  while (defined($l = <STDIN>)) { $l =~ s/</&lt;/gs;  $l =~ s/>/&gt;/gs;  print $l; }


# eof

