#!/usr/local/bin/perl -w

=head1 reindex.pl
    
    History:
        BWK 14 May 00 -- Written; Bill Kelly <billk@cts.com>

    Permission to copy, use, modify, sell and distribute this software
    is granted provided this copyright notice appears in all copies.
    Copyright (C) 2000 Bill Kelly / Full Body Groove.  All Rights Reserved.
    This software is released in the hope it will be useful, but please
    understand you use it at your own risk.
    THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
    WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
    OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

=cut

BEGIN { umask(000) }

use strict;
require FSDBM_File;
require FulltextIndexer_Adaptor;
require porter;  # word stemmer
require qwcfg;  # QwikWiki config
use qwiki qw( &indexWikiPage &validWikiName );

%qwdbm::pages = ();
%qwdbm::searchDb = ();
%qwdbm::search = ();

print "Tying...\n";
tie(%qwdbm::pages, 'FSDBM_File', $qwcfg::pagesDbPath);
tie(%qwdbm::searchDb, 'FSDBM_File', $qwcfg::searchDbPath);
tie(%qwdbm::search, 'FulltextIndexer_Adaptor',
    \%qwdbm::searchDb, 1, $qwcfg::stopwordsFile, \&porter::porter);

print "Locking pages...\n";
tied(%qwdbm::pages)->LockDb();
print "Locking index...\n";
tied(%qwdbm::search)->LockDb();

# Delete existing search indicies
print "Deleting old indicies...\n";
my $docName;
foreach $docName (sort keys %qwdbm::search) {
    print "  $docName\n";
    delete $qwdbm::search{$docName};
}

# Rebuild index
print "Re-indexing pages...\n";
foreach $docName (sort keys %qwdbm::pages) {
    next unless validWikiName($docName);   # weed out metadata files
    print "  $docName\n";
    my $docText = $qwdbm::pages{$docName};
    indexWikiPage($docName, $docText);
}

print "Re-index complete.\n";

tied(%qwdbm::search)->UnlockDb();
tied(%qwdbm::pages)->UnlockDb();

untie(%qwdbm::search);
untie(%qwdbm::searchDb);
untie(%qwdbm::pages);


# eof

